#========================================================================================
# package      DE_EPAGES::MeinPaket::Hooks::Class
# state        public
# description  Downgrade all instances of LineItemMeinPaket
#               and LineItemContainerMeinPaket to LineItemProduct and LineItemContainer.
#========================================================================================
package DE_EPAGES::MeinPaket::Hooks::Class;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );

#========================================================================================
# function     OnDeleteClass
#----------------------------------------------------------------------------------------
# syntax       OnDeleteClass($hParams);
#----------------------------------------------------------------------------------------
# description  When class LineItemMeinPaket is about to be deleted, downgrade all
#               instances to 'LineItemProduct'.
#               Also when 'LineItemContainerMeinPaket' class is about to be deleted,
#               downgrade all instances to 'LineItemContainer'.
#               This allows to uninstall MeinPaket cartridge without invalidating the
#               orders made using MeinPaket.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Object - Class object
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteClass {
    my ($hParams) = @_;

    # AD-8858 Fix MeinPaket uninstall does not work
    _DownGradeToBaseClass($hParams->{'Object'}, 'LineItemProduct');
    _DownGradeToBaseClass($hParams->{'Object'}, 'LineItemContainer');

}


sub _DownGradeToBaseClass() {
    my ($Class, $BaseClassAlias) = @_;

    my $BaseClass = LoadClassByAlias($BaseClassAlias);

    return if $Class->id() == $BaseClass->id();
    return unless $Class->isSubClassOf( $BaseClass );

    my $TargetClass = $Class->baseClass();

    # update the class of all instances
    my $aObjects = $Class->objects();
    $_->set({ 'Class' => $TargetClass }) foreach @$aObjects;
}

1;
