#==============================================================================
# package      DE_EPAGES::Packlink::UI::JSON
# base         DE_EPAGES::Presentation::UI::JSON;
# state        pupblic
#------------------------------------------------------------------------------
# description  JSON handler for Packlink
#==============================================================================
package DE_EPAGES::Packlink::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Packlink::API::AppStorePacklink;
use DE_EPAGES::Packlink::API::Connector;
use DE_EPAGES::Packlink::API::ShippingMethodPacklink;
use DE_EPAGES::Packlink::API::Tools;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use JSON::XS;

use strict;

#==============================================================================
# function     JSONUninstallPackstationApp
# state        public
#------------------------------------------------------------------------------
# syntax       JSONUninstallPackstationApp( $Servlet );
#------------------------------------------------------------------------------
# description  trigger the uninstallation of Packlink.
#------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#==============================================================================
sub JSONUninstallPackstationApp {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShippingProviderMethodPacklinkOrShop = $Servlet->object();
    my $Shop = $ShippingProviderMethodPacklinkOrShop->get('Shop');
    my $redirectUrl = '';
    my $ShippingProviderMethodPacklink;
    my $IsPacklinkAppAvailable = $self->_isPacklinkAppAvailable($Servlet);
    if ( $ShippingProviderMethodPacklinkOrShop->instanceOf('Shop') && $IsPacklinkAppAvailable ) {
        $redirectUrl = BuildShopUrl(
                $ShippingProviderMethodPacklinkOrShop,
                { 'ViewAction' => 'MBO-ViewAppStore' },
                { 'Type' => 'admin', 'UseSSL' => 1 }
        );
        $ShippingProviderMethodPacklink = $self->_getShippingProviderMethodPacklink($Shop);
    }
    else {
        $ShippingProviderMethodPacklink = $ShippingProviderMethodPacklinkOrShop;
    }

    $self->_deleteConnection($ShippingProviderMethodPacklink);
    if ($IsPacklinkAppAvailable) {
        $self->_uninstallPacklinkApp($Servlet, $Shop);
    }
    $self->writeContentOutput( { 'redirectUrl' => $redirectUrl } );

    return;
}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------
sub _deleteConnection {
    my $self = shift;
    my ($ShippingProviderMethodPacklink) = @_;

    TestObject(
            'ShippingProviderMethodPacklink',
            $ShippingProviderMethodPacklink,
            'DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink'
    );
    my $Connector = DE_EPAGES::Packlink::API::Connector->new($ShippingProviderMethodPacklink);
    return unless $Connector->isConnectionSetupDone();

    $Connector->deleteConnection();
    $ShippingProviderMethodPacklink->set( { 'PacklinkID' => undef } );

    return;
}

sub _getShippingProviderMethodPacklink {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingMethodPacklink = DE_EPAGES::Packlink::API::ShippingMethodPacklink->new();
    return $ShippingMethodPacklink->getShippingProviderMethodPacklink($Shop);
}

sub _isPacklinkAppAvailable {
    my $self = shift;
    my ($Servlet) = @_;

    my $Tools = $self->_getTools();
    my $Shop = $Servlet->object()->get('Shop');

    return $Tools->IsPacklinkAppAvailable($Shop);
}

sub _uninstallPacklinkApp {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $AppStorePacklink = $self->_getAppStorePacklink();
    $AppStorePacklink->uninstallPacklinkApp($Servlet, $Shop);

    return;
}

#---------------------------------------------------------------------------
#  Wrappers
#---------------------------------------------------------------------------
sub _getAppStorePacklink {
    my $self = shift;

    return DE_EPAGES::Packlink::API::AppStorePacklink->new();
}

sub _getTools {
    my $self = shift;

    return DE_EPAGES::Packlink::API::Tools->new();
}

1;
