#========================================================================================
# package      DE_EPAGES::Packlink::t::unit::UI_JSON
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for UI::JSON
#========================================================================================
package DE_EPAGES::Packlink::t::unit::UI_JSON;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify::Matcher qw ( HashRef Object );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );
use Test::More;

use strict;

use constant TEST_PACKLINK_ID => '12345';

sub test {
    my $self = shift;

    $self->test_JSONUninstallPackstationApp_from_AppStore();
    $self->test_JSONUninstallPackstationApp_from_ShippingSettings();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_JSONUninstallPackstationApp_from_AppStore {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsPacklinkAppAvailable = 1;
    my $SUT = $self->_createSUT($IsPacklinkAppAvailable);
    my $MockedShop = $self->_createShop();
    my $Servlet = $self->_createServlet($MockedShop);

    $SUT->JSONUninstallPackstationApp($Servlet);

    ok(WasCalled(
            $SUT, '_isPacklinkAppAvailable'), "$SubTestName - Packlink App availablility in AppStore gets called."
    );
    ok(WasCalled(
            $SUT,
            '_getShippingProviderMethodPacklink'),
            "$SubTestName - This is an AppStore deinstall (Servlet object is a Shop)."
    );
    ok(WasCalled($SUT, '_deleteConnection'), "$SubTestName - Packlink connection gets deleted.");
    ok(WasCalled($SUT, '_uninstallPacklinkApp'), "$SubTestName - Packlink App gets uninstalled.");
}

sub test_JSONUninstallPackstationApp_from_ShippingSettings {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsPacklinkAppAvailable = 0;
    my $SUT = $self->_createSUT($IsPacklinkAppAvailable);
    my $MockedShippingProviderMethodPacklink = $self->_createShippingProviderMethodPacklink();
    my $Servlet = $self->_createServlet($MockedShippingProviderMethodPacklink);

    is(
            $MockedShippingProviderMethodPacklink->get('PacklinkID'),
            TEST_PACKLINK_ID,
            "$SubTestName - PacklinkID is set before deinstallation."
    );

    $SUT->JSONUninstallPackstationApp($Servlet);

    ok(WasCalled(
            $SUT, '_isPacklinkAppAvailable'), "$SubTestName - Packlink App availablility in AppStore gets called."
    );
    ok(!WasCalled(
            $SUT,
            '_getShippingProviderMethodPacklink'),
            "$SubTestName - This is a deinstall from Packlink Settings (Servlet object is a ShippingProviderMethodPacklink)."
    );
    is(
            $MockedShippingProviderMethodPacklink->get('PacklinkID'),
            undef,
            "$SubTestName - PacklinkID is undefined after deinstallation."
    );
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($IsPacklinkAppAvailable) = @_;

    my $aParameterList = [];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Packlink::UI::JSON', $aParameterList);

    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
            ->when(
                    Object('DE_EPAGES::Shop::API::Object::Shop'),
                    HashRef({ 'ViewAction' => 'MBO-ViewAppStore' }),
                    HashRef({ 'Type' => 'admin', 'UseSSL' => 1 })
            )->thenReturn('/this/is/redirected/built_url');

    $MockifySut->spy('_isPacklinkAppAvailable')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::Tools::IsPacklinkAppAvailable')
            ->when(
                    Object('DE_EPAGES::Packlink::API::Tools'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($IsPacklinkAppAvailable);

    $MockifySut->spy('_getShippingProviderMethodPacklink')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::ShippingMethodPacklink::getShippingProviderMethodPacklink')
            ->when(
                    Object('DE_EPAGES::Packlink::API::ShippingMethodPacklink'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($self->_createShippingProviderMethodPacklink());

    $MockifySut->spy('_deleteConnection')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::Connector::isConnectionSetupDone')
            ->when(
                    Object('DE_EPAGES::Packlink::API::Connector')
            )->thenReturn(1);
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::Connector::deleteConnection')
            ->when(
                    Object('DE_EPAGES::Packlink::API::Connector')
            )->thenReturnUndef();

    $MockifySut->spy('_uninstallPacklinkApp')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Packlink::API::AppStorePacklink::uninstallPacklinkApp')
            ->when(
                    Object('DE_EPAGES::Packlink::API::AppStorePacklink'),
                    Object(),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturnUndef();

    $MockifySut->mock('writeContentOutput')
            ->whenAny()->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($MockedObjectInServlet) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedObjectInServlet);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($ForInternalShopPointer) = @_;

    my $hAttributes = {
        'instanceOf' => 'Shop'
    };
    $hAttributes->{'Shop'} = $self->_createShop(1) unless ($ForInternalShopPointer);

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyShop->getMockObject();
}

sub _createShippingProviderMethodPacklink {
    my $self = shift;

    my $hAttributes = {
        'instanceOf' => 'ShippingProviderMethodPacklink',
        'PacklinkID' => TEST_PACKLINK_ID,
        'Shop' => $self->_createShop()
    };
    my $MockifyShippingProviderMethodPacklink = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink', $hAttributes
    );

    return $MockifyShippingProviderMethodPacklink->getMockObject();
}


__PACKAGE__->runScript();