#==============================================================================
# package      DE_EPAGES::Shippo::UI::JSON
# base         DE_EPAGES::Presentation::UI::JSON;
# state        pupblic
#------------------------------------------------------------------------------
# description  JSON handler for Shippo
#==============================================================================
package DE_EPAGES::Shippo::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Shippo::API::AppStoreShippo;
use DE_EPAGES::Shippo::API::Connector;
use DE_EPAGES::Shippo::API::ShippingMethodShippo;
use DE_EPAGES::Shippo::API::Tools;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use JSON::XS;

use strict;

#==============================================================================
# function     JSONUninstallApp
# state        public
#------------------------------------------------------------------------------
# syntax       JSONUninstallApp( $Servlet );
#------------------------------------------------------------------------------
# description  trigger the uninstallation of Shippo.
#------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#==============================================================================
sub JSONUninstallApp {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShippingProviderMethodShippoOrShop = $Servlet->object();
    my $Shop = $ShippingProviderMethodShippoOrShop->get('Shop');
    my $redirectUrl = '';
    my $ShippingProviderMethodShippo;
    my $IsShippoAppAvailable = $self->_isShippoAppAvailable($Servlet);
    if ( $ShippingProviderMethodShippoOrShop->instanceOf('Shop') && $IsShippoAppAvailable ) {
        $redirectUrl = BuildShopUrl(
                $Shop, { 'ViewAction' => 'MBO-ViewAppStore' }, { 'Type' => 'admin', 'UseSSL' => 1 }
        );
        $ShippingProviderMethodShippo = $self->_getShippingProviderMethodShippo($Shop);
    }
    else {
        $ShippingProviderMethodShippo = $ShippingProviderMethodShippoOrShop;
    }

    $self->_deleteConnection($ShippingProviderMethodShippo);
    if ($IsShippoAppAvailable) {
        $self->_uninstallShippoApp($Servlet, $Shop);
    }
    $self->writeContentOutput( { 'redirectUrl' => $redirectUrl } );

    return;
}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------
sub _isShippoAppAvailable {
    my $self = shift;
    my ($Servlet) = @_;

    my $Tools = $self->_getTools();
    my $Shop = $Servlet->object()->get('Shop');

    return $Tools->IsShippoAppAvailable($Shop);
}

sub _uninstallShippoApp {
    my $self = shift;
    my ($Servlet, $Shop) = @_;

    my $AppStoreShippo = $self->_getAppStoreShippo();
    $AppStoreShippo->uninstallShippoApp($Servlet, $Shop);

    return;
}

sub _getShippingProviderMethodShippo {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippingMethodShippo = DE_EPAGES::Shippo::API::ShippingMethodShippo->new();
    return $ShippingMethodShippo->getShippingProviderMethodShippo($Shop);
}

sub _deleteConnection {
    my $self = shift;
    my ($ShippingProviderMethodShippo) = @_;

    TestObject(
            'ShippingProviderMethodShippo',
            $ShippingProviderMethodShippo,
            'DE_EPAGES::Shippo::API::Object::ShippingProviderMethodShippo'
    );
    my $Connector = DE_EPAGES::Shippo::API::Connector->new($ShippingProviderMethodShippo);
    return unless $Connector->isConnectionSetupDone();

    $Connector->deleteConnection();
    $ShippingProviderMethodShippo->set( { 'ShippoID' => undef } );

    return;
}

#---------------------------------------------------------------------------
#  Wrappers
#---------------------------------------------------------------------------
sub _getAppStoreShippo {
    my $self = shift;

    return DE_EPAGES::Shippo::API::AppStoreShippo->new();
}

sub _getTools {
    my $self = shift;

    return DE_EPAGES::Shippo::API::Tools->new();
}


1;
