#========================================================================================
# package      DE_EPAGES::Shippo::t::unit::UI_JSON
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for UI::JSON
#========================================================================================
package DE_EPAGES::Shippo::t::unit::UI_JSON;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify::Matcher qw ( HashRef Object );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled );
use Test::More;

use strict;

use constant TEST_SHIPPO_ID => '12345';

sub test {
    my $self = shift;

    $self->test_JSONUninstallApp_from_AppStore();
    $self->test_JSONUninstallApp_from_ShippingSettings();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_JSONUninstallApp_from_AppStore {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShippoAppAvailable = 1;
    my $SUT = $self->_createSUT($IsShippoAppAvailable);
    my $MockedShop = $self->_createShop();
    my $Servlet = $self->_createServlet($MockedShop);

    $SUT->JSONUninstallApp($Servlet);

    ok(WasCalled(
            $SUT, '_isShippoAppAvailable'), "$SubTestName - Shippo App availablility in AppStore gets called."
    );
    ok(WasCalled(
            $SUT,
            '_getShippingProviderMethodShippo'),
            "$SubTestName - This is an AppStore deinstall (Servlet object is a Shop)."
    );
    ok(WasCalled($SUT, '_deleteConnection'), "$SubTestName - Shippo connection gets deleted.");
    ok(WasCalled($SUT, '_uninstallShippoApp'), "$SubTestName - Shippo App gets uninstalled.");
}

sub test_JSONUninstallApp_from_ShippingSettings {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsShippoAppAvailable = 0;
    my $SUT = $self->_createSUT($IsShippoAppAvailable);
    my $MockedShippingProviderMethodShippo = $self->_createShippingProviderMethodShippo();
    my $Servlet = $self->_createServlet($MockedShippingProviderMethodShippo);

    is(
            $MockedShippingProviderMethodShippo->get('ShippoID'),
            TEST_SHIPPO_ID,
            "$SubTestName - ShippoID is set before deinstallation."
    );

    $SUT->JSONUninstallApp($Servlet);

    ok(WasCalled(
            $SUT, '_isShippoAppAvailable'), "$SubTestName - Shippo App availablility in AppStore gets called."
    );
    ok(!WasCalled(
            $SUT,
            '_getShippingProviderMethodShippo'),
            "$SubTestName - This is a deinstall from Shippo Settings (Servlet object is a ShippingProviderMethodShippo)."
    );
    is(
            $MockedShippingProviderMethodShippo->get('ShippoID'),
            undef,
            "$SubTestName - ShippoID is undefined after deinstallation."
    );
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($IsShippoAppAvailable) = @_;

    my $aParameterList = [];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Shippo::UI::JSON', $aParameterList);

    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
            ->when(
                    Object('DE_EPAGES::Shop::API::Object::Shop'),
                    HashRef({ 'ViewAction' => 'MBO-ViewAppStore' }),
                    HashRef({ 'Type' => 'admin', 'UseSSL' => 1 })
            )->thenReturn('/this/is/redirected/built_url');

    $MockifySut->spy('_isShippoAppAvailable')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::Tools::IsShippoAppAvailable')
            ->when(
                    Object('DE_EPAGES::Shippo::API::Tools'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($IsShippoAppAvailable);

    $MockifySut->spy('_getShippingProviderMethodShippo')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::ShippingMethodShippo::getShippingProviderMethodShippo')
            ->when(
                    Object('DE_EPAGES::Shippo::API::ShippingMethodShippo'),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturn($self->_createShippingProviderMethodShippo());

    $MockifySut->spy('_deleteConnection')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::Connector::isConnectionSetupDone')
            ->when(
                    Object('DE_EPAGES::Shippo::API::Connector')
            )->thenReturn(1);
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::Connector::deleteConnection')
            ->when(
                    Object('DE_EPAGES::Shippo::API::Connector')
            )->thenReturnUndef();

    $MockifySut->spy('_uninstallShippoApp')
            ->whenAny();
    $MockifySut->mockStatic('DE_EPAGES::Shippo::API::AppStoreShippo::uninstallShippoApp')
            ->when(
                    Object('DE_EPAGES::Shippo::API::AppStoreShippo'),
                    Object(),
                    Object('DE_EPAGES::Shop::API::Object::Shop')
            )->thenReturnUndef();

    $MockifySut->mock('writeContentOutput')
            ->whenAny()->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($MockedObjectInServlet) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($MockedObjectInServlet);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($ForInternalShopPointer) = @_;

    my $hAttributes = {
        'instanceOf' => 'Shop'
    };
    $hAttributes->{'Shop'} = $self->_createShop(1) unless ($ForInternalShopPointer);

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyShop->getMockObject();
}

sub _createShippingProviderMethodShippo {
    my $self = shift;

    my $hAttributes = {
        'instanceOf' => 'ShippingProviderMethodShippo',
        'ShippoID' => TEST_SHIPPO_ID,
        'Shop' => $self->_createShop()
    };
    my $MockifyShippingProviderMethodShippo = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shippo::API::Object::ShippingProviderMethodShippo', $hAttributes
    );

    return $MockifyShippingProviderMethodShippo->getMockObject();
}


__PACKAGE__->runScript();