#==============================================================================
# package      DE_EPAGES::Shop::UI::Object
# state        private
#==============================================================================
package DE_EPAGES::Shop::UI::Object;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::Error qw( Error GetError ExistsError);
use DE_EPAGES::Shop::API::JWT qw ( AddEditorTokenToURL );

sub NewObject {
    my $self = shift;
    my ($Servlet, @Parameter) = @_;
    my $NewObject;
    eval {
        $NewObject = $self->SUPER::NewObject($Servlet, @Parameter);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'FeatureLimitExceeded') {
            $Servlet->form->mapFormError({
                'Form'=>'FeatureLimitExceeded',
                'Name'=>$Error->vars->{'Name'}
            }, $Error);
            $Servlet->form->executeFormError();
        } else {
            Error($Error);
        }
    }
    return $NewObject;
}

sub SaveShopAddress {
    my $self = shift;
    my ($Servlet) = @_;

    # check if email address is valid
    my $Shop = $Servlet->object;
    $Servlet->form->form($Shop, 'AddressData');

    $self->SUPER::Save($Servlet);

    return;
}

sub UnityViewSFPreview {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object            = $Servlet->object;
    my $Shop              = $Object->getSite();
    my $TargetURLUnitySSL = $Object->get('WebUrlUnitySSL');

    my $PreviewPath = $Servlet->params('PreviewPath');
    $TargetURLUnitySSL .= $PreviewPath if defined $PreviewPath;

    # add token to the end of the URL
    $TargetURLUnitySSL = AddEditorTokenToURL($Servlet, $TargetURLUnitySSL);

    $Servlet->params('TargetURLUnitySSL', $TargetURLUnitySSL);
    $Servlet->params('Title', $Shop->get('NameOrAlias'));

    $self->SUPER::View($Servlet);
    return;
}

sub UnityRedirectToStorefront {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object->getSite();

    return unless $Shop->isUnity();

    $Servlet->redirect($Shop->get('WebUrlUnitySSL'));
}

1;
