#REM
  Parameters:
    #SNIPPET.Type ... string - type of address script (e.g. "Address", "Contact", "Customer")
    #SNIPPET.Context ... string - name of context object (e.g. "Shop", "AddressAttributes") (optional)
    #SNIPPET.Prefix ... string -  prefix for selectors (e.g. "BillingAddress", "ShippingAddress") (optional)
    #SNIPPET.Index ... integer - index to identify similar elements (optional)

  Example:
    #FUNCTION("SNIPPET", "SNIPPET-MenuAddressScriptUS", "Type,Context,Prefix,Index", "Contact", "AddressAttributes", "BillingAddress", #Index)
    #SNIPPET("SNIPPET-MenuAddressScriptUS", "Type,Context,Prefix,Index", "Contact", "AddressAttributes", "BillingAddress", #Index)
#ENDREM

#LOCAL("Prefix", "")
#LOCAL("Context", "")
#LOCAL("Index", "")
    #IF(#DEFINED(#SNIPPET.Prefix))
        #SET("Prefix", #SNIPPET.Prefix)
    #ENDIF

    #IF(#DEFINED(#SNIPPET.Context) AND #SNIPPET.Context NE "")
        #SET("Context", #SNIPPET.Context.".")
    #ENDIF

    #IF(#DEFINED(#SNIPPET.Index))
        #SET("Index", #SNIPPET.Index)
    #ENDIF

    #LOCAL("IsAddressForUSTemplate", #GET(#Context."CountryID") EQ "840")
        var address_840 = {
                #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address,Contact")
                street: {
                    #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address")
                        selector: '#'+'#Prefix#SPACE[0]StreetUS#Index',
                    #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                        selector: '#'+'#Prefix#SPACE[0]Street#Index',
                    #ENDIF
                    value: '#IF(#IsAddressForUSTemplate)#FORMAT(#GET(#Context."Street"), "js")#ENDIF'
                },
                #ENDIF
                #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address")
                street2: {
                    selector: '#'+'#Prefix#SPACE[0]Street2US#Index',
                    value: '#IF(#IsAddressForUSTemplate)#FORMAT(#GET(#Context."Street2"), "js")#ENDIF'
                },
                #ENDIF
                zip: {
                    #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address,Customer")
                        selector: '#'+'#Prefix#SPACE[0]ZipcodeUS#Index',
                    #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                        selector: '#'+'Zipcode#Index',
                    #ENDIF
                    value: '#IF(#IsAddressForUSTemplate)#FORMAT(#GET(#Context."Zipcode"), "js")#ENDIF'
                },
                city: {
                    #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address,Customer")
                        selector: '#'+'#Prefix#SPACE[0]CityUS#Index',
                    #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                        selector: '#'+'#Prefix#SPACE[0]City#Index',
                    #ENDIF
                    value: '#IF(#IsAddressForUSTemplate)#FORMAT(#GET(#Context."City"), "js")#ENDIF'
                },
                state: {
                    #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address,Customer")
                        selector: '#'+'#Prefix#SPACE[0]StateUS#Index',
                    #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                        selector: '#'+'#Prefix#SPACE[0]State#Index',
                    #ENDIF
                    value: '#IF(#IsAddressForUSTemplate)#FORMAT(#GET(#Context."State"), "js")#ENDIF'
                }
            };

        #IF(
                (
                    #DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address"
                    AND
                    (NOT #DEFINED(#Shop.Address.CountryID)) OR (#Shop.Address.CountryID != "840") OR ((#Shop.Address.CountryID == "840") AND (#Shop.CanUseTaxUS == "1" OR #INPUT.TaxUSInvalidAddress == "1"))
                )
                OR
                (
                    #DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact"
                    AND
                    (NOT #DEFINED(#Shop.Address.CountryID)) OR (#Shop.Address.CountryID != "840") OR ((#Shop.Address.CountryID == "840") AND (#Shop.CanUseTaxUS == "1"))
                )
                OR
                (
                    #DEFINED(#SNIPPET.Type) AND NOT (#SNIPPET.Type IN "Address,Contact")
                )
            )
            #LOCAL("CountryNotChangeable", #DEFINED(#GET(#Context."CountryID")) AND #GET(#Context."CountryID") EQ "840")
                #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type EQ "Contact")
                    #SET("CountryNotChangeable", NOT(#InitialShopWizardStartup) AND #DEFINED(#Shop.Address.CountryID) AND #Shop.Address.CountryID EQ "840")
                #ENDIF

                countries['840'] = {
                    template: #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type EQ "Contact") '#ShopWizardSteps-Contact-Tmpl-US' #ELSE '#Menu-#Prefix#SPACE[0]AddressTmpl-US'#ENDIF,
                    address: address_840,
                    #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address")
                    onBeforeRender: function () {
                        #IF(NOT(#CountryNotChangeable))
                            #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address")
                                $('.ep-taxus-addresstmpl-countrynotchangeable-warning').hide();
                            #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                                $('#ShopWizardSteps-Contact-Tmpl-US-CountryNotChangeable-Warning').hide();
                            #ENDIF
                        #ENDIF
                    },
                    #ENDIF
                    onAfterRender: function (address) {
                        var show = (address && address.zip !== undefined && address.zip.value !== undefined && address.zip.value.length) ? true : false;
                        require(['de_epages/taxus/inc/taxus-tools'], function (taxusTools) {
                            var wrapper = $('.#Prefix#SPACE[0]cityAndState');
                            // add functionality to insert a zip code and get the corresponding
                            // county/country/state data for the select field via AJAX
                            taxusTools.wizardAddAddress(
                                $.extend(address_840, {
                                    taxMatrixId: #Shop.Child.TaxMatrixUS.ID,
                                    cityAndState: wrapper,
                                    spanCity: wrapper.find('span.#Prefix#SPACE[0]city'),
                                    spanState: wrapper.find('span.#Prefix#SPACE[0]state'),
                                    show: show,

                                    // callback - executed when an input occurs
                                    onBeforeSelect: function($zip) {
                                        var validateWidget = $zip.data('uiUiValidate');

                                        // if zip input is an uiValidate widget and is already initialized
                                        if (validateWidget && $zip.data('initialized')) {
                                            // set state of zip input invalid
                                            validateWidget._setValidStatus(false);
                                        }

                                        // manually mark zip input as initialized
                                        $zip.data('initialized', true);
                                    },
                                    // callback - executed after an entry was selected from the autocomplete list
                                    onAfterSelect: function($zip) {
                                        var validateWidget = $zip.data('uiUiValidate');

                                        if (validateWidget) {
                                            // set state of zip input valid
                                            validateWidget._setValidStatus(true);
                                        }
                                    },
                                    // additional event handler - executed when the form.submit event triggers
                                    onSubmit: function(event, data) {
                                        var $zip = data.zip,
                                            autocompleteWidget = $zip.data('uiAutocomplete');

                                        // if autocomplete widget exists and is invalid
                                        if (autocompleteWidget && $zip.hasClass('ui-invalid')) {
                                            // remove hidden inputs and display suggest list
                                            data.setHiddenInputs();

                                            // display autocomplete list if a search term exists
                                            if (autocompleteWidget.term !== undefined) {
                                                autocompleteWidget.menu.activeMenu.show();
                                            } else {
                                                if (!autocompleteWidget.NOT_VALID) {
                                                    $zip.autocomplete('search', $zip.val());
                                                }
                                            }

                                            // hide suggest list if invalid input
                                            if (autocompleteWidget && autocompleteWidget.NOT_VALID) {
                                              autocompleteWidget.menu.activeMenu.hide();
                                            }
                                        }
                                    },
                                    // additional event handler - executed when the form.submit event triggers
                                    onFocus: function(event, data) {
                                        var $zip = data.zip,
                                            autocompleteWidget = $zip.data('uiAutocomplete');

                                        // if autocomplete widget exists and is invalid
                                        if (autocompleteWidget && $zip.hasClass('ui-invalid')) {

                                            // display autocomplete list if a search term exists
                                            if (autocompleteWidget.term !== undefined) {
                                                autocompleteWidget.menu.activeMenu.show();
                                            }

                                            // hide suggest list if invalid input
                                            if (autocompleteWidget && autocompleteWidget.NOT_VALID) {
                                              autocompleteWidget.menu.activeMenu.hide();
                                            }
                                        }
                                    }
                                })
                            );
                            #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address")
                                #IF(NOT #DEFINED(#INPUT.TaxUSInvalidAddress))$('#'+'CanUseTaxUS').val(1);#ENDIF
                            #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                                // set TaxModel input to 0
                                $('input[type="hidden"][name="TaxModel"]').val(0);
                                $('#'+'CanUseTaxUS').val(1);
                            #ENDIF
                        });

                        #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address")
                            #IF(NOT(#CountryNotChangeable))
                                $('.ep-taxus-addresstmpl-countrynotchangeable-warning').show();
                            #ENDIF
                        #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                            #IF(NOT(#CountryNotChangeable) AND NOT(#InitialShopWizardStartup))
                                $('#ShopWizardSteps-Contact-Tmpl-US-CountryNotChangeable-Warning').show();
                            #ENDIF
                        #ENDIF
                    }
                };

                #IF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Address")
                    var countrySelect = $('#' + 'ep-#ID#SPACE[0]_CountryID');

                    #IF(#CountryNotChangeable)
                        countrySelect
                            .prop('disabled', true)
                            .hide(0);
                        countrySelect.before($('<span class="state ep-input-style-disabled">' + countrySelect[0].options[countrySelect[0].selectedIndex].text + '</span>'));
                    #ELSE
                        $('#ep-taxus-tooltip-wrapper').replaceWith($('#Menu-AddressTmpl-US-CountryNotChangeable').tmpl());
                        $('#Menu-AddressTmpl-US-CountryNotChangeable-Tooltip').metaparse();
                    #ENDIF
                #ELSIF(#DEFINED(#SNIPPET.Type) AND #SNIPPET.Type IN "Contact")
                    var countrySelect = $('#' + 'CountryID#Index');

                    #IF(#CountryNotChangeable)
                        countrySelect
                            .prop('disabled', true)
                            .hide(0);
                        countrySelect.before($('<span style="width: 160px; float: left; margin-bottom: 10px;" class="ep-input-style-disabled">' + countrySelect[0].options[countrySelect[0].selectedIndex].text + '</span>'));
                    #ELSE
                        countrySelect.before($('#ShopWizardSteps-Contact-Tmpl-US-CountryNotChangeable').tmpl());
                        $('#ShopWizardSteps-Contact-Tmpl-US-CountryNotChangeable-Tooltip').metaparse();
                    #ENDIF
                #ENDIF

            #ENDLOCAL #REM /* CountryNotChangeable */ #ENDREM
        #ENDIF

    #ENDLOCAL #REM /* IsAddressForUSTemplate */ #ENDREM

#ENDLOCAL #REM /* selectedIndex */ #ENDREM
#ENDLOCAL #REM /* Context */ #ENDREM
#ENDLOCAL #REM /* Prefix */ #ENDREM
